#!/usr/bin/python
# Terminator by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""gnomeconfig.py - a gateway to Gnome defaults"""

from util import dbg

try:
    import gconf
except ImportError:
    dbg('Unable to import gconf, GNOME defaults unavailable')

class GnomeConfig(object):
    """
    A caching, read-only proxy to GConf variables.

    This will subscribe to GConf notifications should the values change at runtime.
    (This with the hope one doesn't have to restart Terminator to apply changes)
    """

    def __init__(self):
        self.gconf = None
        self.cache = {}
        if 'gconf' in globals():
            self.gconf.client_get_default()

    def get(self, path):
        """
        Returns None if GConf is not available.
        Returns the GConf value for path otherwise.
        When path leads nowhere, not sure what happens (XXX).
        """
        if self.gconf is None:
            return None
        if path in self.cache:
            return self.cache[path]
        self.cache[path] = self.gconf.get(path)
        self.gconf.notify_add(path, self.on_gconf_notify)
        return self.cache[path]

    def get_string(self, path):
        """
        A convenience wrapper around get, equivalent to calling get(path).get_string()
        Returns None when get()
        """
        value = self.get(path)
        if value is None:
            return None
        return value.get_string()

    def on_gconf_notify(self, client, timestamp, entry, *extra):
        key = entry.get_key()
        value = entry.get_value()

        # assert key in self.cache
        self.cache[key] = value
